<?php

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

class GOL_IBE_Search_Form_settings {
	const OPTIONS_ID = 'GOL_IBE_settings';
	private static $instance = null;
	public $options;
	private $default_settings = [
		'background_color' => '#ffffff',
		'secondary_background_color' => '#eeeeee',
		'background_transparency' => '100',
		'button_color' => '#df5829',
		'button_text_color' => 'white',
		'domain' => 'example.com',
		'show_hotels' => false,
		'language' => 'en',
		'text_color' => 'black'
	];

	public static function get_instance() {
		if ( null == self::$instance ) {
			self::$instance = new self;
		}
		return self::$instance;
	}

	private function __construct() {
		add_action('admin_menu', [&$this, 'add_page']);
		add_action('admin_init', [&$this, 'register_page_options']);
		add_action('admin_enqueue_scripts', [$this, 'enqueue_admin_js']);
		$this->options = $this->load_options();
	}

	private function load_options() {
		$settings = get_option(self::OPTIONS_ID);
		if (FALSE === $settings) {
			$settings = $this->default_settings;
			add_option(self::OPTIONS_ID, $settings);
		}
		return $settings;
	}

	public function add_page() {
		add_options_page('GOL IBE Search Form Settings', 'GOL IBE Search Form', 'manage_options', __FILE__, [$this, 'display_page']);
	}

	public function display_page() {
		?>
		<div class="wrap">
		    <h1>GOL IBE Search Form Setting</h1>
			<form method="post" action="options.php">
			<?php
				settings_fields(__FILE__);
				do_settings_sections(__FILE__);
				submit_button();
			?>
			</form>
		</div> <!-- /wrap -->
		<hr />
		<p>Please add [iforma-golibe] shortcode anywhere, where you want use this plugin.</p>
		<?php
	}

	public function register_page_options() {
		add_settings_section('golibe_general_section', 'General', [$this, 'display_section'], __FILE__);
		add_settings_field('golibe_domain', 'Domain', [$this, 'domain_field'], __FILE__, 'golibe_general_section' );
		add_settings_field('golibe_show_hotels', 'Show Hotels', [$this, 'show_hotels_field'], __FILE__, 'golibe_general_section' );
		add_settings_field('golibe_language', 'Language', [$this, 'language_field'], __FILE__, 'golibe_general_section' );
		add_settings_section('golibe_design_section', 'Design', [$this, 'display_section'], __FILE__);
		add_settings_field('golibe_background_color', 'Background Color', [$this, 'background_color_field'], __FILE__, 'golibe_design_section' );
		add_settings_field('golibe_secondary_background_color', 'Secondary Background Color', [$this, 'secondary_background_color_field'], __FILE__, 'golibe_design_section' );
		add_settings_field('golibe_background_transparency', 'Background Transparency', [$this, 'background_transparency_field'], __FILE__, 'golibe_design_section' );
		add_settings_field('golibe_text_color', 'Text Color', [$this, 'text_color_field'], __FILE__, 'golibe_design_section' );
		add_settings_field('golibe_button_color', 'Buttons Color', [$this, 'button_color_field'], __FILE__, 'golibe_design_section' );
		add_settings_field('golibe_button_text_color', 'Buttons Text Color', [$this, 'button_text_color_field'], __FILE__, 'golibe_design_section' );
		register_setting( __FILE__, self::OPTIONS_ID, [$this, 'validate_options']);
	}

	public function background_color_field() {
		echo '<input type="color" name="' . self::OPTIONS_ID . '[background_color]" value="' . sanitize_hex_color($this->get_option('background_color')) . '" />';
	}

	public function secondary_background_color_field() {
		echo '<input type="color" name="' . self::OPTIONS_ID . '[secondary_background_color]" value="' . sanitize_hex_color($this->get_option('secondary_background_color')) . '" />';
	}

	public function background_transparency_field() {
		$val = sanitize_text_field($this->get_option('background_transparency'));
		echo '<input type="range" id="background_transparency" name="' . self::OPTIONS_ID . '[background_transparency]" min="0" max="100" value="' . $val . '" />';
		echo '<input type="text" id="background_transparency_value" value="' . $val . '%" disabled=disabled />';
	}

	public function button_color_field() {
		echo '<input type="color" name="' . self::OPTIONS_ID . '[button_color]" value="' . sanitize_hex_color($this->get_option('button_color')) . '" />';
	}

	public function domain_field() {
		echo '<input type="text" name="' . self::OPTIONS_ID . '[domain]" value="' . sanitize_text_field($this->get_option('domain')) . '" />';
	}

	public function show_hotels_field() {
		echo '<input type="checkbox" name="' . self::OPTIONS_ID . '[show_hotels]" value="1" ' . checked(1, $this->get_option('show_hotels'), false) . ' />';
	}

	public function language_field() {
		$langs = require "langs.php";
		$selected_lang = $this->get_option('language');
		echo '<select name="' . self::OPTIONS_ID . '[language]">';
		foreach ($langs as $key => $value) {
			echo '<option value="' . $key . '" ' . selected($key, $selected_lang, false) . '>' . $value . '</option>';
		}
		echo '</select>';
	}

	public function button_text_color_field() {
		$val = $this->get_option('button_text_color');
		echo '<select name="' . self::OPTIONS_ID . '[button_text_color]">';
		echo '<option value="black" ' . selected($val, 'black', false) . '>black</option>';
		echo '<option value="white" ' . selected($val, 'white', false) . '>white</option>';
		echo '</select>';
	}

	public function text_color_field() {
		$val = $this->get_option('text_color');
		echo '<select name="' . self::OPTIONS_ID . '[text_color]">';
		echo '<option value="black" ' . selected($val, 'black', false) . '>black</option>';
		echo '<option value="white" ' . selected($val, 'white', false) . '>white</option>';
		echo '</select>';
	}

    private function get_option($name) {
		return isset($this->options[$name]) ? $this->options[$name] : $this->default_settings[$name];
	}

	public function validate_options($fields) {
		$valid_fields = [];

		$background_color = $this->check_color($fields['background_color']);
		if ($background_color) {
			$valid_fields['background_color'] = $background_color;
		} else {
			$valid_fields['background_color'] = $this->options['background_color'];
			add_settings_error(self::OPTIONS_ID, 'gitlab_background_color_error', 'Insert a valid color for Background Color', 'error');
		}

		$secondary_background_color = $this->check_color($fields['secondary_background_color']);
		if ($secondary_background_color) {
			$valid_fields['secondary_background_color'] = $secondary_background_color;
		} else {
			$valid_fields['secondary_background_color'] = $this->options['secondary_background_color'];
			add_settings_error(self::OPTIONS_ID, 'gitlab_secondary_background_color_error', 'Insert a valid color for Secondary Background Color', 'error');
		}

		$background_transparency = $this->check_transparency($fields['background_transparency']);
		if ($background_transparency !== false) {
			$valid_fields['background_transparency'] = $background_transparency;
		} else {
			$valid_fields['background_transparency'] = $this->options['background_transparency'];
			add_settings_error(self::OPTIONS_ID, 'gitlab_background_transparency_error', 'Insert a valid number for Background Transparency', 'error');
		}

		$button_color = $this->check_color($fields['button_color']);
		if ($button_color) {
			$valid_fields['button_color'] = $button_color;
		} else {
			$valid_fields['button_color'] = $this->options['button_color'];
			add_settings_error(self::OPTIONS_ID, 'gitlab_button_color_error', 'Insert a valid color for Buttons Color', 'error');
		}

		$domain = $this->check_domain($fields['domain']);
		if ($domain) {
			$valid_fields['domain'] = $domain;
		} else {
			$valid_fields['domain'] = $this->options['domain'];
			add_settings_error(self::OPTIONS_ID, 'gitlab_domain_error', 'Insert a valid domain for Domain', 'error');
		}

        if ($fields['show_hotels'] === '1') {
			$valid_fields['show_hotels'] = true;
		} else {
			$valid_fields['show_hotels'] = false;
		}

		$language = $this->check_language($fields['language']);
		if ($language) {
			$valid_fields['language'] = $language;
		} else {
			$valid_fields['language'] = $this->options['language'];
			add_settings_error(self::OPTIONS_ID, 'gitlab_language_error', 'Select a valid language for Language', 'error');
		}

		$text_color = $this->check_bw_color($fields['text_color']);
		if ($text_color) {
			$valid_fields['text_color'] = $text_color;
		} else {
			$valid_fields['text_color'] = $this->options['text_color'];
			add_settings_error(self::OPTIONS_ID, 'gitlab_text_color_error', 'Select a valid color for Text Color', 'error');
		}

		$button_text_color = $this->check_bw_color($fields['button_text_color']);
		if ($button_text_color) {
			$valid_fields['button_text_color'] = $button_text_color;
		} else {
			$valid_fields['button_text_color'] = $this->options['button_text_color'];
			add_settings_error(self::OPTIONS_ID, 'gitlab_button_text_color_error', 'Select a valid color for Buttons Text Color', 'error');
		}

		return $valid_fields;
	}

    private function check_bw_color($value) {
		$sanitized = sanitize_key($value);
		if (in_array($sanitized, ['black', 'white'])) {
			return $sanitized;
		} else {
			return false;
		}
	}

    private function check_language($value) {
		$sanitized = sanitize_key($value);
		$langs = require "langs.php";
		if (key_exists($sanitized, $langs)) {
			return $sanitized;
		} else {
			return false;
		}
	}

	private function check_domain($value) {
		$sanitized = sanitize_text_field($value);
		if (preg_match('/^(?:(?:(?<thld>[\w\-]*)(?:\.))?(?<sld>[\w\-]*))\.(?<tld>\w*)(?:\:(?<port>\d*))?$/m', $sanitized)) {
			return $sanitized;
		} else {
			return false;
		}
	}

    private function check_transparency($value) {
		$sanitized = sanitize_text_field($value);
		if (preg_match('/^(100|[1-9][0-9]|[0-9])$/m', $sanitized)) {
			return $sanitized;
		} else {
			return false;
		}
	}

	private function check_color($value) {
		$sanitized = sanitize_hex_color($value);
		if (preg_match('/^#(?:[0-9a-fA-F]{3}){1,2}$/m', $sanitized)) {
			return $sanitized;
		} else {
			return false;
		}
	}

	public function display_section() { /* Leave blank */ }

	public function enqueue_admin_js() {
		wp_enqueue_script( 'cpa_custom_js', plugins_url( 'GOL-IBE-Search-Form.js', __FILE__ ), array( 'jquery', 'wp-color-picker' ), '', true);
	}

}

GOL_IBE_Search_Form_settings::get_instance();

/* vim:set autoindent noexpandtab tabstop=4 shiftwidth=4: */
