<?php
/**
 * @package GOL-IBE-Search-Form
 * @version 1.1.2
 */
/*
Plugin Name: GOL IBE Search Form
Description: Search Form form for adding GOL IBE search form to your sites.
Author: CEE Group Travelport a.s.
Version: 1.1.2
*/

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

require "GOL-IBE-Search-Form-settings.php";
require "GOL-IBE-Search-Form-CSS.php";

class GOL_IBE_Search_Form {
	private static $instance = null;
	private $settings;
	private $configId;
	private $customizedStyles;

	public static function get_instance($settings) {
		if ( null == self::$instance ) {
			self::$instance = new self($settings, new GOL_IBE_Search_Form_CSS);
		}
		return self::$instance;
	}

	private function __construct($settings, $customizedStyles) {
		$this->settings = $settings;
		$this->configId = 'GOL_IBE_config_' . sanitize_key($settings['language']);
		$this->customizedStyles = $customizedStyles;
	}

    public function setup() {
		add_action('wp_enqueue_scripts', [$this, 'setting_up_scripts']);
		add_filter('cron_schedules', [$this, 'add_cron_recurrence_interval']);
		add_action('golibe_cron_hook', [$this, 'cron_exec']);

		if (!wp_next_scheduled('golibe_cron_hook')) {
			wp_schedule_event(time(), 'golibe_daily', 'golibe_cron_hook');
		}

		add_shortcode('iforma-golibe', [$this, 'main']);

	}

	public function main() {
		if ($this->settings['domain'] == "example.com") {
			if (current_user_can('manage_options')) {
				return '<h1 style="color: red">Travelport plugin is not set yet. Please go to the wp-admin and configure it now.</h1>';
			} else { return ""; }
		}

		$config = $this->load_config();

		$path = plugin_dir_path(__FILE__) . "content/index." . $this->settings['language'] . ".html";

		if (file_exists($path)) {
			$html = file_get_contents($path);

			preg_match('/<div id="ao3SearchForm" class="container">.*<\/div>/s', $html, $forms);

			if (isset($forms[0])) {
				wp_enqueue_style('golibe-gol-flights-icons');
				wp_enqueue_style('golibe-jquery-ui-core');
				wp_enqueue_style('golibe-jquery-ui-datepicker');
				wp_enqueue_style('golibe-jquery-ui-autocomplete');
				wp_enqueue_style('golibe-jquery-ui-menu');
				wp_enqueue_style('golibe-jquery-ui-selectmenu');
				wp_enqueue_style('golibe-autocomplete');
				wp_enqueue_style('golibe-googlemapslink');
				wp_enqueue_style('golibe-flights');
				wp_enqueue_style('iforma-golibe-css-override');

				wp_enqueue_script('jquery');
				wp_enqueue_script('jquery-ui-core');
				wp_enqueue_script('jquery-ui-datepicker');
				wp_enqueue_script('jquery-ui-widget');
				wp_enqueue_script('jquery-ui-dialog');
				wp_enqueue_script('jquery-ui-button');
				wp_enqueue_script('jquery-ui-position');
				wp_enqueue_script('jquery-ui-autocomplete');
				wp_enqueue_script('jquery-ui-autocomplete-html');
				wp_enqueue_script('jquery-ui-menu');
				wp_enqueue_script('jquery-ui-selectmenu');
				wp_enqueue_script('golibe-forall-destinationchoose-ajax');
				wp_enqueue_script('golibe-flights');
				wp_enqueue_script('golibe-forall-destinationwhisper-ajax');
				$form = $forms[0];
				$form = str_replace("{{domain}}", "https://" . $this->settings['domain'], $form);
				if ($this->settings['show_hotels']) {
					$form = str_replace('j-redirectToHotels hidden', 'j-redirectToHotels', $form);
				}
				$css = $this->customizedStyles->render(
						$this->settings['background_color'],
						$this->settings['secondary_background_color'],
						$this->settings['background_transparency'],
						$this->settings['button_color'],
						$this->settings['text_color'],
						$this->settings['button_text_color']
				);
				$preset = 'window.GOL_Global = window.GOL_Global || {};window.GOL_Global.config = window.GOL_Global.config || {};';
				return "<style>{$css}</style><script>{$preset}{$config}</script><div id=\"travelport\">{$form}</div>";
			} else {
				if (current_user_can('manage_options')) {
					return '<h1 style="color: red">Travelport plugin is probably broken. Please contact plugin provider about that incident.</h1>';
				} else { return ""; }
			}
		} else {
			if (current_user_can('manage_options')) {
				return '<h1 style="color: red">Travelport plugin installation is probably broken. Please try reinstall it and if that problem persists, contact plugin provider about that incident.</h1>';
			} else { return ""; }
		}
	}

	public function setting_up_scripts() {
		wp_register_style('golibe-gol-flights-icons', plugin_dir_url( __FILE__ ) . 'content/css/gol-flights-icons.css');
		wp_register_style('golibe-jquery-ui-core', plugin_dir_url( __FILE__ ) . 'content/css/05-jqui-core.css');
		wp_register_style('golibe-jquery-ui-datepicker', plugin_dir_url( __FILE__ ) . 'content/css/08-jqui-datepicker.css');
		wp_register_style('golibe-jquery-ui-autocomplete', plugin_dir_url( __FILE__ ) . 'content/css/40-jqui-autocomplete.css');
		wp_register_style('golibe-jquery-ui-menu', plugin_dir_url( __FILE__ ) . 'content/css/41-jqui-menu.css');
		wp_register_style('golibe-jquery-ui-selectmenu', plugin_dir_url( __FILE__ ) . 'content/css/42-jqui-selectmenu.css');
		wp_register_style('golibe-autocomplete', plugin_dir_url( __FILE__ ) . 'content/css/52-autocomplete.css');
		wp_register_style('golibe-googlemapslink', plugin_dir_url( __FILE__ ) . 'content/css/56-googleMapsLink.css');
		wp_register_style('golibe-flights', plugin_dir_url( __FILE__ ) . 'content/css/flights.css');
		wp_register_style('golibe-datepicker-icons', plugin_dir_url( __FILE__ ) . 'content/css/datepicker-icons.css');

		wp_register_script('jquery-ui-autocomplete-html', plugin_dir_url( __FILE__ ) . 'content/js/jquery.ui.autocomplete.html.js');
		wp_register_script('golibe-forall-destinationchoose-ajax', plugin_dir_url( __FILE__ ) . 'content/js/ForAll.DestinationChoose.ajax.js');
		wp_register_script('golibe-flights', plugin_dir_url( __FILE__ ) . 'content/js/flights.js');
		wp_register_script('golibe-forall-destinationwhisper-ajax', plugin_dir_url( __FILE__ ) . 'content/js/ForAll.DestinationWhisper.ajax.js');
	}

	public function add_cron_recurrence_interval($schedules) {
		$schedules['golibe_daily'] = array(
				'interval'  => 86400,
				'display'   => __( 'Executes travelport\'s WP plugin once day', 'textdomain' )
				);
		return $schedules;
	}

	public function cron_exec() {
		$this->download_config();
	}

    private function load_config() {
		$config = get_option($this->configId);
		if (FALSE === $config) {
			add_option($this->configId, []);
			$this->download_config();
			$config = get_option($this->configId);
		}
		return $config;
	}

    public function error_notice_download_domain_not_set() {
		echo '<div class="error notice"><p>Config for GOL-IBE-Search plugin was not downloaded, because domain was not set yet.</p></div>';
	}

    public function error_notice_download_domain_not_responding() {
		echo '<div class="error notice"><p>Config for GOL-IBE-Search plugin was not downloaded, because domain "'.sanitize_text_field($this->settings['domain']).'" not responding.</p></div>';
	}

	private function download_config() {
		$domain = $this->settings['domain'];
		if ($domain == "example.com") {
			add_action('admin_notices', [$this, error_notice_download_domain_not_set]);
			return false;
		}

		$code = $this->settings['language'];
		$url = "https://{$domain}/index.php?action=fHTMLConfig&lang={$code}";
		$auth = base64_encode("gol:log");
		$context = stream_context_create([
			"http" => [
				"header" => "Authorization: Basic $auth"
			]
		]);
		if ($jsfile = file_get_contents($url, false, $context)) {
			update_option($this->configId, $jsfile);
		} else {
			add_action('admin_notices', [$this, error_notice_download_domain_not_responding]);
			return false;
		}
		return true;
	}
}

GOL_IBE_Search_Form::get_instance(GOL_IBE_Search_Form_settings::get_instance()->options)->setup();

/* vim:set autoindent noexpandtab tabstop=4 shiftwidth=4: */
