window.GOL_Global = window.GOL_Global || {};
window.GOL_Global.config = window.GOL_Global.config || {};
window.GOL_Global.ForAll = window.GOL_Global.ForAll || {};

window.GOL_Global.ForAll.DestinationChoose = function(connectorType) {
  var $ = jQuery;

  var getSegmentIndex = window.GOL_Global.ForAll.getSegmentIndex;
  var countryList = [];
  var destinationCache = {};

  function loadCountryList() {
      function refreshCountryList(data) {
          if (data && data.length) {
              countryList = data;
              renderCountryOptions(data);
          }
      }

      var url = getCurrentUrl() + 'json/country-list/';
      $.ajax({
          url: url,
          dataType: 'jsonp',
          data: {
              language: GOL_Global.config.lang
          },
          success: refreshCountryList
      });
  }

  function loadDestinationList(country, selectedDestinationCode) {
      if (country in destinationCache) {
          renderDestinationOptions(destinationCache[country]);
          selectDestination(selectedDestinationCode);
          return;
      }

      function refreshDestinationList(data) {
          if (data && data.length) {
              destinationCache[country] = data;
              renderDestinationOptions(data);
              selectDestination(selectedDestinationCode);
          } else {
              renderDestinationOptions([]);
          }
      }
      var url = getCurrentUrl() + 'json/destination-list/';
      $.ajax({
          url: url,
          dataType: 'jsonp',
          data: {
              language: GOL_Global.config.lang,
              type: connectorType,
              country: country
          },
          success: refreshDestinationList
      });
  }

  function renderOptions(list, label, select) {
      select.empty();
      $('<option>').attr({
          'value': '',
          'selected': 'selected',
          'disabled': 'disabled'
      }).text(label).appendTo(select);
      var len = list.length;
      for (var i = 0; i < len; i++) {
          var item = list[i];
          if (item.code && item.name) {
              $('<option>').attr('value', item.code).text(item.name).prop('destData', item).appendTo(select);
          }
      }
      select.attr('disabled', len === 0);
      select.selectmenu('refresh');
  }

  function renderCountryOptions(countries) {
      renderOptions(countries, GOL_Global.config.textCountry, $('.j-chooseCountry'));
  }

  function renderSubCountryOptions(subCountries) {
      renderOptions(subCountries, GOL_Global.config.textSubCountry, $('.j-chooseSubCountry'));
  }

  function renderDestinationOptions(destinations) {
      renderOptions(destinations, GOL_Global.config.textLocation, $('.j-choosePlace'));
  }

  function getSubCountries(countryCode) {
      var country = $.grep(countryList, function(obj) {
          return obj.code === countryCode;
      });
      if (country.length > 0 && 'states' in country[0]) {
          return country[0].states;
      } else {
          return [];
      }
  }

  function countryChange() {
      var countryCode = $(this).val();
      if (countryCode) {
          renderSubCountryOptions(getSubCountries(countryCode));
          loadDestinationList(countryCode, '');
      } else {
          renderSubCountryOptions([]);
          refreshDestinationList([]);
      }
  }

  function subCountryChange() {
      var subCountryCode = $(this).val();
      if (subCountryChange) {
          loadDestinationList(subCountryCode, '');
      } else {
          refreshDestinationList([]);
      }
  }

  function setSelectValue(select, value) {
      $('option', select).each(function() {
          this.selected = this.value === value;
      });
      select.selectmenu('refresh');
  }

  function selectCountry(countryCodes) {
      setSelectValue($('.j-chooseCountry'), countryCodes[0]);
      if (countryCodes.length === 2) {
          renderSubCountryOptions(getSubCountries(countryCodes[0]));
          setSelectValue($('.j-chooseSubCountry'), countryCodes.join('_'));
      } else {
          renderSubCountryOptions([]);
      }
  }

  function selectDestination(destCode) {
      setSelectValue($('.j-choosePlace'), destCode);
  }

  function setupForm(countryCode, destinationCode) {

      var countryCodes = countryCode.split('_');
      selectCountry(countryCodes);
      loadDestinationList(countryCode, destinationCode);
  }

  function init() {
      loadCountryList();
      $('.j-chooseOriginButton').click(function() {
          choosePlaceDialog(getSegmentIndex(this), 'j-flightOrigin', 'j-flightOriginCode');
      });
      $('.j-chooseDestinationButton').click(function() {
          choosePlaceDialog(getSegmentIndex(this), 'j-flightDestination', 'j-flightDestinationCode');
      });

      function choosePlaceDialog(segmentIndex, destinationInputClass, destinationCodeInputClass) {
          var destinationInput = $('.' + destinationInputClass + '[data-segment-index="' + segmentIndex + '"]');
          var destinationCodeInput = $('.' + destinationCodeInputClass + '[data-segment-index="' + segmentIndex + '"]');
          var destData = destinationCodeInput.prop('destData');
          selectCountry(['']);
          renderSubCountryOptions([]);
          renderDestinationOptions([]);
          if (destData && destData.length > 0) {
              setupForm(destData[0].country.code, destData[0].code);
          }
          $('.j-choosePlace').off('selectmenuchange change').on('selectmenuchange change', function() {
              destinationInput.val($(this).val());
              if (this.selectedIndex > -1) {
                  var destData = this.options[this.selectedIndex].destData;
                  var destLabel = destData.name + ' (' + destData.code + ')';
                  destinationInput.val(destLabel);
                  destinationCodeInput.val(destData.code).prop({
                      destData: [destData],
                      searchValue: destLabel
                  });
              }
          });
          $('.j-chooseDestination').dialog({
              appendTo: '#ao3SearchForm',
              modal: true,
              position: {
                  my: "left top",
                  at: "left bottom",
                  of: destinationInput
              },
              buttons: [{
                  text: "Ok",
                  click: function() {
                      $(this).dialog('close');
                  }
              }]
          }).dialog('open');
      }
      $('.j-chooseCountry').on('selectmenuchange change', countryChange);
      $('.j-chooseSubCountry').on('selectmenuchange change', subCountryChange);
  }

  init();
}
