/*
* jQuery UI Autocomplete HTML Extension
*
* Copyright 2010, Scott González (http://scottgonzalez.com)
* Dual licensed under the MIT or GPL Version 2 licenses.
*
* http://github.com/scottgonzalez/jquery-ui-extensions
*/
(function($) {

  var proto = $.ui.autocomplete.prototype,
      initSource = proto._initSource;

  function filter(array, term) {
      var matcher = new RegExp($.ui.autocomplete.escapeRegex(term), "i");
      return $.grep(array, function(value) {
          return matcher.test($("<div>").html(value.label || value.value || value).text());
      });
  }

  function createGoogleLinkElement(destName, country) {
      var googleSearchTerm = 'Airports ' + destName + ' ' + country;
      return $("<div class='googleMapsLink j-googleMapsLink' title='" + googleSearchTerm + "' data='" + googleSearchTerm + "'></div>");
  }

  $.extend(proto, {
      _initSource: function() {
          if (this.options.html && $.isArray(this.options.source)) {
              this.source = function(request, response) {
                  response(filter(this.options.source, request.term));
              };
          } else {
              initSource.call(this);
          }
      },

      _renderItem: function(ul, item) {
          var googleLinkElement = this.options.enableGoogleMapsLink && item.value.length !== 0 ? createGoogleLinkElement(item.destName, item.country) : $();
          return $("<li></li>")
              .data("item.autocomplete", item)
              .append($("<a></a>")[this.options.html ? "html" : "text"](item.label).append(googleLinkElement))
              .addClass(item.className)
              .appendTo(ul);
      },

      _normalize: function(items) {
          return items;
      },

      _renderMenu: function(ul, items) {
          var that = this;
          $.each(items, function(index, item) {
              that._renderItemData(ul, item);
          });
          if (this.options.enableGoogleMapsLink) {
              $(ul).addClass('enableGoogleMapsLink');
          }
      }
  });

})(jQuery);
